@@ -12,22 +12,20 @@ using namespace std;
 class Nonterminal_Rule : public Rule
 {
 public:
-  virtual Nonterminal_Rule* Clone() const = 0;
-
   virtual ~Nonterminal_Rule();
 
   virtual void Reset_String();
 
   virtual const bool Check_For_String();
-  virtual const list<string> Get_String() const;
+  virtual const list<string>& Get_String() const;
 
   virtual const Rule* operator[](const unsigned int in_index) const;
 
 protected:
   Nonterminal_Rule();
-  Nonterminal_Rule(const Nonterminal_Rule &in_nonterminal);
 
-  virtual const Nonterminal_Rule& operator= (const Nonterminal_Rule &in_nonterminal);
+private:
+  Nonterminal_Rule(const Nonterminal_Rule &in_nonterminal);
 
 protected:
   list<Rule_List*> m_rule_lists;
