@@ -138,25 +138,9 @@ EOF
     {
       $OUT .=<<EOF;
 
-  $return_type Get_Value() const
+  const $return_type Get_Value() const
   {
-EOF
-      if (defined $nonpointer_return_type)
-      {
-        $OUT .=<<EOF;
-    // We return a copy when dollar_dollar is a pointer, since this function
-    // is called multiple times
-    return new $nonpointer_return_type(*dollar_dollar);
-EOF
-      }
-      else
-      {
-        $OUT .=<<EOF;
-    return $return_type(dollar_dollar);
-EOF
-      }
-
-    $OUT .=<<EOF;
+    return dollar_dollar;
   }
 EOF
 
@@ -169,48 +153,33 @@ EOF
 ]]]
 // ---------------------------------------------------------------------------
 
-[[[$nonterminal]]]* [[[$nonterminal]]]::Clone() const
-{
-  return new [[[$nonterminal]]](*this);
-}
-
-#ifndef DISABLE_GENERATED_STRING_CACHING_OPTIMIZATION
-// ---------------------------------------------------------------------------
-
 [[[$nonterminal]]]::[[[$nonterminal]]]() : Nonterminal_Rule()
 {
-}
+[[[
+  foreach my $i (1..$#productions+1)
+  {
+    $OUT .= "  m_$i = NULL;\n";
+  }
+]]]}
 
 // ---------------------------------------------------------------------------
 
-[[[$nonterminal]]]::[[[$nonterminal]]](const [[[$nonterminal]]] &in_[[[$nonterminal]]]) : Nonterminal_Rule(in_[[[$nonterminal]]])
+[[[$nonterminal]]]::~[[[$nonterminal]]]()
 {
-  *this = in_[[[$nonterminal]]];
-}
+[[[
+  foreach my $i (1..$#productions+1)
+  {
+    $OUT .= <<EOF;
+  if (m_$i != NULL)
+    delete m_$i;
+EOF
+  }
+]]]}
 
 // ---------------------------------------------------------------------------
 
-const [[[$nonterminal]]]& [[[$nonterminal]]]::operator= (const [[[$nonterminal]]] &in_[[[$nonterminal]]])
-{
-  Nonterminal_Rule::operator=(in_[[[$nonterminal]]]);
-
-  m_first_cache_string = in_[[[$nonterminal]]].m_first_cache_string;
-  m_using_cache = in_[[[$nonterminal]]].m_using_cache;
-
-  return *this;
-}
-
-#endif // DISABLE_GENERATED_STRING_CACHING_OPTIMIZATION
-// ---------------------------------------------------------------------------
-
 void [[[$nonterminal]]]::Initialize(const unsigned int in_allowed_length, const Rule *in_previous_rule)
 {
-  list<Rule_List*>::iterator a_rule_list;
-  for(a_rule_list = m_rule_lists.begin();
-      a_rule_list != m_rule_lists.end();
-      a_rule_list++)
-    delete *a_rule_list;
-
   m_rule_lists.clear();
 [[[
   foreach my $i (1..$#productions+1)
@@ -223,7 +192,12 @@ void [[[$nonterminal]]]::Initialize(const unsigned int in_allowed_length, const
 #ifndef DISABLE_PRODUCTION_LENGTH_OPTIMIZATION
   if (in_allowed_length $operator $number)
 #endif // DISABLE_PRODUCTION_LENGTH_OPTIMIZATION
-    m_rule_lists.push_back(new match_$i);
+  {
+    if (m_$i == NULL)
+      m_$i = new match_$i;
+
+    m_rule_lists.push_back(m_$i);
+  }
 EOF
   }
 
@@ -236,7 +210,7 @@ EOF
 
 // ---------------------------------------------------------------------------
 
-$return_type ${nonterminal}::Get_Value() const
+const $return_type ${nonterminal}::Get_Value() const
 EOF
     $OUT .= "{\n";
 
@@ -263,6 +237,7 @@ EOF
     $OUT .= "}\n";
   }
 
+  chomp $OUT;
 ]]]
 // ---------------------------------------------------------------------------
 
@@ -372,7 +347,7 @@ const bool [[[$nonterminal]]]::Check_For_String()
 
 // ---------------------------------------------------------------------------
 
-const list<string> [[[$nonterminal]]]::Get_String() const
+const list<string>& [[[$nonterminal]]]::Get_String() const
 {
 #ifndef DISABLE_GENERATED_STRING_CACHING_OPTIMIZATION
   // We check m_using_cache in case this terminal was in the middle of
