@@ -3,7 +3,7 @@
 
 using namespace std;
 
-#include "generator/utility/utility.h"
+#include "model/utility/nonterminal_utility.h"
 #include "generator/rule/nonterminal_rule.h"
 #include "generator/rule/terminal_rule.h"
 
@@ -12,83 +12,160 @@ using namespace std;
 class IDENTIFIER : public Terminal_Rule
 {
 public:
-  virtual const bool Check_For_String()
-  {
-    if (!Is_Valid())
-      return false;
+  virtual const bool Check_For_String();
+  virtual const list<string>& Get_String() const;
+  virtual const string& Get_Value() const;
 
-    static map<unsigned int, unsigned int> counts;
-
-    if (counts.find(m_string_count) != counts.end())
-    {
-      if (counts[m_string_count] == 1)
-        counts.erase(m_string_count);
-      else
-        counts[m_string_count]--;
-    }
-
-    m_string_count++;
-
-    if (m_string_count <= counts.size() + 1)
-    {
-      counts[m_string_count]++;
-      return true;
-    }
-    else
-      return false;
-  }
-
-  virtual const list<string> Get_String() const
-  {
-    stringstream temp_stream;
-
-    temp_stream << "id_" << m_string_count;
-
-    list<string> strings;
-    strings.push_back(temp_stream.str());
-    return strings;
-  }
+protected:
+  list<string> strings;
+  string return_value;
 };
 
 // ---------------------------------------------------------------------------
 
+const bool IDENTIFIER::Check_For_String()
+{
+  if (!Is_Valid())
+    return false;
+
+  static map<unsigned int, unsigned int> counts;
+
+  if (counts.find(m_string_count) != counts.end())
+  {
+    if (counts[m_string_count] == 1)
+      counts.erase(m_string_count);
+    else
+      counts[m_string_count]--;
+  }
+
+  m_string_count++;
+
+  if (m_string_count > counts.size() + 1)
+    return false;
+
+  counts[m_string_count]++;
+
+  stringstream temp_stream;
+
+  temp_stream << "id_" << m_string_count;
+  return_value = temp_stream.str();
+
+  strings.clear();
+
+  strings.push_back(return_value);
+
+  return true;
+}
+
+// ---------------------------------------------------------------------------
+
+const list<string>& IDENTIFIER::Get_String() const
+{
+  return strings;
+}
+
+// ---------------------------------------------------------------------------
+
+const string& IDENTIFIER::Get_Value() const
+{
+  return return_value;
+}
+
+// ---------------------------------------------------------------------------
+// ---------------------------------------------------------------------------
+
 class identifier_list : public Nonterminal_Rule
 {
+  class match_1;
+  class match_2;
+
 public:
-  virtual void Initialize(const unsigned int in_allowed_length, const Rule *in_previous_rule = NULL)
+  identifier_list();
+  virtual ~identifier_list();
+
+  virtual void Initialize(const unsigned int in_allowed_length, const Rule *in_previous_rule = NULL);
+
+protected:
+  match_1 *m_1;
+  match_2 *m_2;
+};
+
+// ---------------------------------------------------------------------------
+
+class identifier_list::match_1 : public Rule_List
+{
+  friend class identifier_list;
+
+  match_1()
   {
-    list<Rule_List*>::iterator a_rule_list;
-    for(a_rule_list = m_rule_lists.begin();
-        a_rule_list != m_rule_lists.end();
-        a_rule_list++)
-      delete *a_rule_list;
-
-    m_rule_lists.clear();
-
-    {
-      Rule_List* rule_list = new Rule_List;
-
-      rule_list->push_back(new IDENTIFIER);
-
-      m_rule_lists.push_back(rule_list);
-    }
-
-    {
-      Rule_List* rule_list = new Rule_List;
-
-      rule_list->push_back(new identifier_list);
-      rule_list->push_back(new IDENTIFIER);
-
-      m_rule_lists.push_back(rule_list);
-    }
-
-    Nonterminal_Rule::Initialize(in_allowed_length,in_previous_rule);
+    push_back(new IDENTIFIER);
   }
 
 };
 
 // ---------------------------------------------------------------------------
 
+class identifier_list::match_2 : public Rule_List
+{
+  friend class identifier_list;
+
+  match_2()
+  {
+    push_back(new identifier_list);
+    push_back(new IDENTIFIER);
+  }
+
+};
+
+// ---------------------------------------------------------------------------
+
+identifier_list::identifier_list() : Nonterminal_Rule()
+{
+  m_1 = NULL;
+  m_2 = NULL;
+}
+
+// ---------------------------------------------------------------------------
+
+identifier_list::~identifier_list()
+{
+  if (m_1 != NULL)
+    delete m_1;
+  if (m_2 != NULL)
+    delete m_2;
+}
+
+// ---------------------------------------------------------------------------
+
+void identifier_list::Initialize(const unsigned int in_allowed_length, const Rule *in_previous_rule)
+{
+  m_rule_lists.clear();
+
+#ifndef DISABLE_PRODUCTION_LENGTH_OPTIMIZATION
+  if (in_allowed_length == 1)
+#endif // DISABLE_PRODUCTION_LENGTH_OPTIMIZATION
+  {
+    if (m_1 == NULL)
+      m_1 = new match_1;
+
+    m_rule_lists.push_back(m_1);
+  }
+
+#ifndef DISABLE_PRODUCTION_LENGTH_OPTIMIZATION
+  if (in_allowed_length >= 1)
+#endif // DISABLE_PRODUCTION_LENGTH_OPTIMIZATION
+  {
+    if (m_2 == NULL)
+      m_2 = new match_2;
+
+    m_rule_lists.push_back(m_2);
+  }
+
+  Nonterminal_Rule::Initialize(in_allowed_length, in_previous_rule);
+}
+
+// ---------------------------------------------------------------------------
+
 int main(int argc, char *argv[])
 {
   list< list< string > > results;
@@ -100,10 +177,7 @@ int main(int argc, char *argv[])
 
     while(start.Check_For_String())
     {
-      list<string> generated_string = start.Get_String();
-//      cout << "GENERATED STRING: " <<
-//        Utility::to_string(generated_string.begin(),generated_string.end()) << endl;
-      results.push_back( generated_string );
+      results.push_back( start.Get_String() );
     }
   }
 
